// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file SolverMatrix.h */

#ifndef AMDIS_SOLVERMATRIX_H
#define AMDIS_SOLVERMATRIX_H

#include "DOFMatrix.h"

namespace AMDiS {

  /**
   * \brief Helper class to provide complete matrix for ITL solvers and preconditioners
   */

  // General case undefined !
  template <typename Matrix>
  class SolverMatrix { };


  // DOFMatrix, i.e. ScalarProblem
  template <>
  class SolverMatrix<DOFMatrix>
  {
    public : 
      SolverMatrix() : matrix(0) {}

      void setMatrix(const DOFMatrix& A)
      {
	matrix= &A.getBaseMatrix();
      }

      const DOFMatrix::base_matrix_type& getMatrix() const 
      { 
	return *matrix; 
      }

    private:
      const DOFMatrix::base_matrix_type* matrix;
  };


  // VectorProblem
  template <>
  class SolverMatrix<Matrix<DOFMatrix*> >
  {
    public : 
      void setMatrix(const Matrix<DOFMatrix*>& A)
      {
	int                 ns= A.getSize();
	std::vector<int>    block_starts(ns+1);
	block_starts[0]= 0;
	for (int i= 0; i < ns; ++i)
	  block_starts[i+1]= block_starts[i] + A[i][i]->getFESpace()->getAdmin()->getUsedSize();

	matrix.change_dim(block_starts[ns], block_starts[ns]);
	set_to_zero(matrix);

	int nnz= 0;
	for (int rb= 0; rb < ns; ++rb)
	  for (int cb= 0; cb < ns; ++cb)
	    if (A[rb][cb])
		nnz+= A[rb][cb]->getBaseMatrix().nnz();	  
      
	DOFMatrix::inserter_type  ins(matrix, int(1.2 * nnz / matrix.dim1()));
	for (int rb= 0; rb < ns; ++rb)
	  for (int cb= 0; cb < ns; ++cb)
	    if (A[rb][cb])
	      ins[block_starts[rb]][block_starts[cb]] << A[rb][cb]->getBaseMatrix();	  
      }

      const DOFMatrix::base_matrix_type& getMatrix() const 
      { 
	return matrix; 
      }

    private:      
       DOFMatrix::base_matrix_type matrix;
  };



} // namespace AMDiS

#endif // AMDIS_SOLVERMATRIX_H
